"""
A transformation that propagates zeros. It transforms:
Indexed(Zero((shape),(),()) -> Zero((),(free_indices),(index_dimensions))
"""

from dune.perftool.ufl.transformations import ufl_transformation

from ufl.algorithms import MultiFunction
from ufl.classes import Zero


class ZeroPropagation(MultiFunction):
    call = MultiFunction.__call__

    def __call__(self, expr):
        # self.replacemap = GetIndexMap()(expr)
        return self.call(expr)

    def expr(self, o):
        return self.reuse_if_untouched(o, *tuple(self.call(op) for op in o.ufl_operands))

    def indexed(self, o):
        op, i = o.ufl_operands
        if isinstance(op, Zero):
            return Zero(shape=o.ufl_shape,
                        free_indices=o.ufl_free_indices,
                        index_dimensions=o.ufl_index_dimensions)
        else:
            return self.reuse_if_untouched(o, self.call(op), self.call(i))


@ufl_transformation(name='zero')
def zero_propagation(expr):
        return ZeroPropagation()(expr)
